
#include <Arduino.h>

//Automaton and custom machines
#include <Automaton.h>
#include "Atm_Teenstep.h"
#include "Atm_Teenstep_OSC.h"

//osc
#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <TeensyMAC.h>
#include <OSCBundle.h>


// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network:
byte mac[] = {
 0x00, 0xAA, 0xBB, 0xCC, 0xDE, 0x02
};

IPAddress ip(192, 168, 1, 104);    //local IP of Arduino/Teensy
unsigned int localPort = 8000;      // local port to listen on (not needed for multicast)

IPAddress outIp(192, 168, 1, 102);
const unsigned int outPort = 9000;
// IPAddress ipMulti(239, 200, 200, 200); // ipMidi Multicast address
// unsigned int portMulti = 9977;   // ipMidi Mutlicast port 1


// buffers for receiving and sending data
byte packetBuffer[UDP_TX_PACKET_MAX_SIZE];       // buffer to hold incoming packet
byte sendBuffer1[] = {0x90, 0x14, 0x22};        // MIDI Note On to Multicast address
byte sendBuffer2[] = {0x80, 0x14, 0x00};        // MIDI Note Off to Multicast address

// An EthernetUDP instance to let us send and receive packets over UDP
EthernetUDP Udp;

///////////////////       OSC         ////////////////////////
OSCBundle bndl;

/////////////////// STEPPER machines  ////////////////////////
const int HIGH_SPEED = 800 ;
const int HIGH_ACC = 800 ;
const int LOW_SPEED = 100 ;
const int LOW_ACC = 1000 ;

Atm_Teenstep_OSC A_low_OSC;
Atm_Teenstep A_low_step;
Stepper A_low_stepper(8 , 7);
StepControl A_low_controller ;

Atm_Teenstep_OSC B_low_OSC;
Atm_Teenstep B_low_step;
Stepper B_low_stepper(32 , 31);
StepControl B_low_controller ;

Atm_Teenstep_OSC TrBr_low_OSC;
Atm_Teenstep TrBr_low_step;
Stepper TrBr_low_stepper(2 , 1);
StepControl TrBr_low_controller ;

Atm_Teenstep_OSC LevBr_low_OSC;
Atm_Teenstep LevBr_low_step;
Stepper LevBr_low_stepper(5 , 4);
StepControl LevBr_low_controller ;


/////////////////// Servo and pump  ////////////////////////

const int filling_pwm_pin = 14;
const int brush_pwm_pin = 20;
const int crusher_pin = 16;


void motorsOSC(OSCMessage &msg){
  msg.dispatch("/filling", [](OSCMessage &msg){analogWrite(filling_pwm_pin, msg.getFloat(0));});
  msg.dispatch("/brush", [](OSCMessage &msg){analogWrite(brush_pwm_pin, msg.getInt(0));});
  msg.dispatch("/crusher", [](OSCMessage &msg){digitalWrite(crusher_pin, msg.getInt(0));});
}

//////////////  Setup  /////////////////////

void setup() {
 SPI.setSCK(27);
 Ethernet.init(10);//(10)
 teensyMAC(mac);
 Ethernet.begin(mac, ip);              // start the Ethernet and UDP:
 // Udp.beginMulticast(ipMulti, portMulti); // for modified Teensy Ethernet library
  Udp.begin(localPort);

 //  SERIAL
 Serial.begin(115200); // higher Baud rate for faster refresh, CHANGE IN SERIAL MONITOR
 delay(2000);
 Serial.println("Started Low");

 //  STEPPERS
 A_low_step.trace( Serial );
 A_low_step.begin(A_low_stepper, A_low_controller)
           .setEnablePin(6)//.enableReversed(1);//.enable(1);
           .setLimitType(3).setLimitPins(A16).limitReversed(false).limitThresholds(600, 750, 950, 1200);
 A_low_stepper.setMaxSpeed(HIGH_SPEED);
 A_low_stepper.setAcceleration(HIGH_ACC);
 A_low_stepper.setInverseRotation(true);
 A_low_OSC.begin(A_low_step, Udp, bndl, "/A_low");
 A_low_step.onChangeposition([](int idx, int v, int up){bndl.add("/A_low_OSC/step").add(v);});
 A_low_step.onChange([](int idx, int v, int up){bndl.add("/A_low_OSC/state").add(v);});
 A_low_step.onLimitlow([](int idx, int v, int up){bndl.add("/A_low_OSC/limitLow").add(v);});
 A_low_step.onLimithigh([](int idx, int v, int up){bndl.add("/A_low_OSC/limitHigh").add(v);});

 B_low_step.trace( Serial );
 B_low_step.begin(B_low_stepper, B_low_controller)
           .setEnablePin(30)//.enableReversed(1);//.enable(1);
           .setLimitType(3).setLimitPins(A14).limitReversed(false).limitThresholds(600, 750, 950, 1200);
 B_low_stepper.setMaxSpeed(HIGH_SPEED);
 B_low_stepper.setAcceleration(HIGH_ACC);
 B_low_stepper.setInverseRotation(true);
 B_low_OSC.begin(B_low_step, Udp, bndl, "/B_low");
 B_low_step.onChangeposition([](int idx, int v, int up){bndl.add("/B_low_OSC/step").add(v);});
 B_low_step.onChange([](int idx, int v, int up){bndl.add("/B_low_OSC/state").add(v);});
 B_low_step.onLimitlow([](int idx, int v, int up){bndl.add("/B_low_OSC/limitLow").add(v);});
 B_low_step.onLimithigh([](int idx, int v, int up){bndl.add("/B_low_OSC/limitHigh").add(v);});

 TrBr_low_step.trace( Serial );
 TrBr_low_step.begin(TrBr_low_stepper, TrBr_low_controller)
           .setEnablePin(0)//.enableReversed(1);//.enable(1);
           .setLimitType(2).setLimitPins(17, 15).limitReversed(true).limitThresholds(600, 750, 950, 1200);
 TrBr_low_stepper.setMaxSpeed(HIGH_SPEED);
 TrBr_low_stepper.setAcceleration(HIGH_ACC);
 TrBr_low_stepper.setInverseRotation(true);
 TrBr_low_OSC.begin(TrBr_low_step, Udp, bndl, "/TrBr_low");
 TrBr_low_step.onChangeposition([](int idx, int v, int up){bndl.add("/TrBr_low_OSC/step").add(v);});
 TrBr_low_step.onChange([](int idx, int v, int up){bndl.add("/TrBr_low_OSC/state").add(v);});
 TrBr_low_step.onLimitlow([](int idx, int v, int up){bndl.add("/TrBr_low_OSC/limitLow").add(v);});
 TrBr_low_step.onLimithigh([](int idx, int v, int up){bndl.add("/TrBr_low_OSC/limitHigh").add(v);});

 LevBr_low_step.trace( Serial );
 LevBr_low_step.begin(LevBr_low_stepper, LevBr_low_controller)
           .setEnablePin(3)//.enableReversed(1);//.enable(1);
           .setLimitType(2).setLimitPins(39, 37).limitReversed(true).limitThresholds(600, 750, 950, 1200);
 LevBr_low_stepper.setMaxSpeed(HIGH_SPEED);
 LevBr_low_stepper.setAcceleration(HIGH_ACC);
 LevBr_low_stepper.setInverseRotation(true);
 LevBr_low_OSC.begin(LevBr_low_step, Udp, bndl, "/LevBr_low");
 LevBr_low_step.onChangeposition([](int idx, int v, int up){bndl.add("/LevBr_low_OSC/step").add(v);});
 LevBr_low_step.onChange([](int idx, int v, int up){bndl.add("/LevBr_low_OSC/state").add(v);});
 LevBr_low_step.onLimitlow([](int idx, int v, int up){bndl.add("/LevBr_low_OSC/limitLow").add(v);});
 LevBr_low_step.onLimithigh([](int idx, int v, int up){bndl.add("/LevBr_low_OSC/limitHigh").add(v);});

 //  Peristaltic pump
 pinMode(filling_pwm_pin, OUTPUT);
 pinMode(brush_pwm_pin, OUTPUT);
 pinMode(crusher_pin, OUTPUT);

}

void loop() {
  automaton.run();
  // Udp.beginPacket(ipMulti, portMulti);
  Udp.beginPacket(outIp, outPort);
  bndl.send(Udp); // send the bytes to the SLIP stream
  Udp.endPacket(); // mark the end of the OSC Packet
  bndl.empty(); // empty the bundle to free room for a new one

  OSCMessage msgIn;
  int size;
  if( (size = Udp.parsePacket())>0)
  {
    while(size--)
      msgIn.fill(Udp.read());

     if(!msgIn.hasError()){
       Serial.println("got OSC");
       A_low_OSC.onOSC(msgIn);
       B_low_OSC.onOSC(msgIn);
       TrBr_low_OSC.onOSC(msgIn);
       LevBr_low_OSC.onOSC(msgIn);
       motorsOSC(msgIn);

     }
  }
}
